"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.NineHentaiAPI = void 0;
const tslib_1 = require("tslib");
const node_fetch_1 = tslib_1.__importDefault(require("node-fetch"));
const APITagType_1 = require("./api-types/APITagType");
const NineHentaiBook_1 = require("./NineHentaiBook");
const xtend_1 = tslib_1.__importDefault(require("xtend"));
const abort_controller_1 = tslib_1.__importDefault(require("abort-controller"));
const NineHentaiSearch_1 = require("./NineHentaiSearch");
const escape_string_regexp_1 = tslib_1.__importDefault(require("escape-string-regexp"));
const tag_aliases_json_1 = tslib_1.__importDefault(require("./tag_aliases.json"));
const sorted_tag_aliases = Object.entries(tag_aliases_json_1.default).sort(([a], [b]) => b.length - a.length);
const default_options = {
    fetch_options: {},
    timeout: 25000,
    fetch_tags: true,
    tag_minimum_usage: 20
};
const default_search_options = {
    page: 0,
    pages: { range: [0, 2000] },
    sort: 0 /* NEWEST */,
    text: '',
    tag: {
        items: {
            excluded: [],
            included: []
        }
    }
};
class NineHentaiAPI {
    constructor(options) {
        this.options = options;
        this.options = xtend_1.default(default_options, options);
        if (!this.options.fetch_tags || NineHentaiAPI.tags.length)
            return;
        Object.values(APITagType_1.APITagType).filter(Number).forEach((tag_type) => {
            NineHentaiAPI.tags.push(new Promise(async (resolve) => {
                const result = await this.get_tags({
                    type: tag_type,
                    search: {
                        letter: '',
                        page: 0,
                        sort: 1 /* MOST_USES */,
                        text: '',
                        uses: this.options.tag_minimum_usage
                    }
                });
                if (!result.status || !result.total_count)
                    return resolve();
                const tag_requests = [...new Array(result.total_count - 1).keys()].map(page => {
                    return this.get_tags({
                        type: tag_type,
                        search: {
                            letter: '',
                            page: page + 1,
                            sort: 1 /* MOST_USES */,
                            text: '',
                            uses: this.options.tag_minimum_usage
                        }
                    }).then(result => result.status && result.results);
                });
                return resolve([result.results, ...(await Promise.all(tag_requests))]);
            }));
        });
    }
    static async get_all_tags() {
        return (await Promise.all(NineHentaiAPI.tags)).flat(Infinity);
    }
    get_fetch_options(options) {
        var _a, _b, _c;
        return Object.assign({}, this.options.fetch_options, options, {
            headers: Object.assign({}, (_b = (_a = this.options.fetch_options) === null || _a === void 0 ? void 0 : _a.headers) !== null && _b !== void 0 ? _b : {}, (_c = options.headers) !== null && _c !== void 0 ? _c : {})
        });
    }
    async get_tags(request) {
        const controller = new abort_controller_1.default();
        const timeout = setTimeout(() => controller.abort(), this.options.timeout);
        const result = await node_fetch_1.default('https://9hentai.com/api/getTags', this.get_fetch_options({
            method: 'POST',
            body: JSON.stringify(request),
            headers: {
                'content-type': 'application/json;charset=UTF-8',
                'accept': 'application/json'
            },
            signal: controller.signal
        }));
        clearTimeout(timeout);
        return result.json();
    }
    get_book(book_or_id) {
        if (typeof book_or_id == 'number' || typeof book_or_id == 'string') {
            return this.fetch_book_details_by_id(book_or_id).then(api_book => {
                return api_book && new NineHentaiBook_1.NineHentaiBook(api_book);
            });
        }
        return new NineHentaiBook_1.NineHentaiBook(book_or_id);
    }
    async fetch_book_details_by_id(id) {
        const book = await this.get_book_by_id(id);
        if (!book.status)
            return null;
        let page = 0;
        let total_pages = Infinity;
        while (page < total_pages) {
            const book_search = await this.api_search({
                page: page,
                pages: { range: [0, 2000] },
                sort: 0 /* NEWEST */,
                text: `"${book.results.title}"`,
                tag: {
                    items: { excluded: [], included: [] }
                }
            });
            if (!book_search.status)
                break;
            total_pages = book_search.total_count;
            const book_details = book_search.results.find(book => book.id == id);
            if (book_details)
                return book_details;
            ++page;
        }
        return null;
    }
    async get_book_by_id(id) {
        const controller = new abort_controller_1.default();
        const timeout = setTimeout(() => controller.abort(), this.options.timeout);
        const result = await node_fetch_1.default('https://9hentai.com/api/getBookByID', this.get_fetch_options({
            method: 'POST',
            body: `{"id":${id}}`,
            headers: {
                'content-type': 'application/json;charset=UTF-8',
                'accept': 'application/json'
            },
            signal: controller.signal
        }));
        clearTimeout(timeout);
        return result.json();
    }
    async api_search(request) {
        const controller = new abort_controller_1.default();
        const timeout = setTimeout(() => controller.abort(), this.options.timeout);
        const result = await node_fetch_1.default('https://9hentai.com/api/getBook', this.get_fetch_options({
            method: 'POST',
            body: JSON.stringify({ search: request }),
            headers: {
                'content-type': 'application/json;charset=UTF-8',
                'accept': 'application/json'
            },
            signal: controller.signal
        }));
        clearTimeout(timeout);
        return result.json();
    }
    get_tags_from_search(query, tags) {
        const output_tags = new Set();
        query = query.replace(/\s+/g, ' ');
        tags.sort((a, b) => b.name.length - a.name.length).forEach(tag => {
            const tag_regex = new RegExp(`\\b${escape_string_regexp_1.default(tag.name)}\\b`, 'ig');
            if (!tag_regex.test(query))
                return;
            query = query.replace(tag_regex, '');
            output_tags.add(tag);
        });
        sorted_tag_aliases.forEach(([alias, [type, id]]) => {
            const tag_regex = new RegExp(`\\b${escape_string_regexp_1.default(alias)}\\b`, 'ig');
            if (!tag_regex.test(query))
                return;
            query = query.replace(tag_regex, '');
            output_tags.add({
                id: id,
                type: type
            });
        });
        return [query.replace(/\s+/g, ' '), [...output_tags]];
    }
    async search(query_or_options, options) {
        var _a;
        let search_options;
        if (typeof query_or_options == 'string') {
            search_options = xtend_1.default(default_search_options, options !== null && options !== void 0 ? options : {});
            let query = query_or_options;
            if ((_a = options === null || options === void 0 ? void 0 : options.auto_tag) !== null && _a !== void 0 ? _a : true) {
                const tags = (await Promise.all(NineHentaiAPI.tags)).flat(Infinity);
                let output_tags = [];
                [query, output_tags] = this.get_tags_from_search(query, tags);
                search_options.tag.items.included = output_tags;
            }
            search_options.text = query;
        }
        else {
            search_options = xtend_1.default(default_search_options, query_or_options);
        }
        return new NineHentaiSearch_1.NineHentaiSearch(await this.api_search(search_options));
    }
}
exports.NineHentaiAPI = NineHentaiAPI;
NineHentaiAPI.tags = [];
//# sourceMappingURL=NineHentaiAPI.js.map